<?php
/**
 * Proforma_Driver:: defines an API for implementing storage backends for Proforma.
 *
 * $Horde: proforma/lib/Driver.php,v 1.13 2006/03/07 18:17:34 slusarz Exp $
 *
 * Copyright 2004-2006 Duck <duck@obala.net>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @package Proforma
 */
class Proforma_Driver {

    /**
     * Array holding the current foo list. Each array entry is a hash
     * describing a foo. The array is indexed by the IDs.
     *
     * @var array
     */
    var $_foos = array();

    /**
     * Lists all foos.
     *
     * @return array  Returns a list of all foos.
     */
    function listFoos()
    {
        return $this->_foos;
    }

    /**
     * Attempts to return a concrete Proforma_Driver instance based on $driver.
     *
     * @param string $driver  The type of the concrete Proforma_Driver subclass
     *                        to return.  The class name is based on the
     *                        storage driver ($driver).  The code is
     *                        dynamically included.
     *
     * @param array $params   A hash containing any additional configuration
     *                        or connection parameters a subclass might need.
     *
     * @return Proforma_Driver  The newly created concrete Proforma_Driver
     *                          instance, or false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Proforma_Driver_' . $driver;
        if (class_exists($class)) {
            $proforma = &new $class($params);
        } else {
            $proforma = false;
        }

        return $proforma;
    }

    /**
     * Attempts to return a reference to a concrete Proforma_Driver instance
     * based on $driver. It will only create a new instance if no
     * Proforma_Driver instance with the same parameters currently exists.
     *
     * This method must be invoked as: $var = &Proforma_Driver::singleton()
     *
     * @param string $driver  The type of concrete Proforma_Driver subclass to
     *                        return.  The is based on the storage driver
     *                        ($driver).  The code is dynamically included.
     * @param array $params   A hash containing any additional configuration
     *                        or connection parameters a subclass might need.
     *
     * @return mixed  The created concrete Proforma_Driver instance, or false
     *                on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Proforma_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
