<?php
/**
 * Proforma Base Class.
 *
 * Copyright 2004-2006 Duck <duck@obala.net>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @package Proforma
 */
class Proforma {

    /**
     * Invice types
     *
     * TODO Implement permission to invoice type
     *
     * @return array client list
     */
    function invoiceTypes()
    {
        $types = array('normal' => _("Invoice"),
                       'proforma' => _("Proforma"));
        return $types;
    }

    /**
     * Build clients lists
     *
     * @param string $type list type
     *
     * @return array client list
     */
    function listClients($type = false)
    {
        static $clients;

        if (is_null($clients)) {
            $args = array('addresses' => array(''),
                          'fields' => array('name'));
            $result = $GLOBALS['registry']->call('clients/searchClients', $args);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $clients = array();
            if (!empty($result)) {
                $result = $result[''];
                foreach ($result as $client) {
                    $clients[$client['id']] = $client;
                }
            }
        }

        switch ($type) {
            case 'tiny':
                foreach ($clients as $id => $client) {
                    $data['id'] = $client['name'];
                }
            break;
            default:
                $data = $clients;
            break;        
        }

        return $data;
    }

    /**
     * Build client attributes data
     *
     * @param string $type list type
     *
     * @return array attribute list
     */
    function clientsAttributes($type = 'plain')
    {
        static $clientsAttributes;

        if (is_null($clientsAttributes)) {
            $source = $GLOBALS['registry']->call('clients/getClientSource');
            $clientsAttributes = $GLOBALS['registry']->call('clients/fields', $source);
        }

        switch ($type) {
            case 'labes':
                $attributes = array();
                foreach ($clientsAttributes as $field) {
                    $attributes[$field['name']] = $field['label'];
                }
            break;
            default:
                return $attributes = $clientsAttributes;
            break;
        }

        return $attributes;
    }

    /**
     * Build api item list
     *
     * @param string $api   Application to check
     *
     * @return array    Items from listCostObjects method
     */
    function externalItems($api)
    {
        global $registry;
    
        return $registry->call($registry->applications[$api]['provides'] . '/listCostObjects',
                               array(array()));
    }

    /**
     * External api list
     *
     * Modules from where we can get items have listCostObjects method
     *
     * @return array of api and its names
     */
    function externalApps()
    {
        global $registry;

        static $apps;

        if (is_null($apps)) {
            $apps = $GLOBALS['registry']->listApps(null, true);
            foreach ($apps as $api) {
                if (isset($registry->applications[$api]['provides']) && 
                    $registry->hasMethod($registry->applications[$api]['provides'] . '/listCostObjects')) {
                    $apps[$api] = $registry->applications[$api]['name'];
                } else {
                    unset($apps[$api]);
                }
            }
        }
        
        return $apps;
    }

    /**
     * Differecne in dates
     *
     * @param string $from  starting date
     * @param string $to    ending date
     *
     * @return intiger days to go
     */
    function diffDates($from, $to = false)
    {
        $to = ($to) ? strtotime($to) : time();
        $from = strtotime($from);
        $diff = ($from-$to)/86400;

        return ceil($diff);
    }

    /**
     * Differecne in dates
     *
     * @param string $from  starting date
     *
     * @return string html to output
     */
    function taxEnum($selected = '')
    {
        global $taxes;

        $html = '';
        foreach ($taxes as $tax) {
            $html .= '<option value="' . $tax['value'] . '"';
            if ($selected === $tax['name']) {
                ' selected="selected" ';
            }
            $html .= '>' . $tax['name'] . '</option>' . "\n";
        }
        
        return $html;
    }

    /**
     * Differecne in dates
     *
     * @param string $from  starting date
     *
     * @return string html to output
     */
    function formatCurrency($value, $currency = 0)
    {
        global $currencies;

        $currency = $currencies[0];
        $html = $currency['symbol_left'];
        $html .= number_format($value*$currency['value'], $currency['decimal_places'], $currency['decimal_point'], $currency['thousands_point']);
        $html .= $currency['symbol_right'];

        return $html;
    }

    /**
     * Build Proforma's list of menu items.
     */
    function getMenu($returnType = 'object')
    {
        global $conf, $registry, $browser, $print_link;

        require_once 'Horde/Menu.php';

        $menu = &new Menu(HORDE_MENU_MASK_ALL);
        $menu->add(Horde::applicationUrl('list.php'), _("List"), 'calendar.png', $registry->getImageDir('horde'));
        $menu->add(Horde::applicationUrl('invoice.php'), _("Invoice"), 'edit.png', $registry->getImageDir('horde'));
        $menu->add(Horde::applicationUrl('clients.php'), _("Clients"), 'group.png', $registry->getImageDir('horde'));
        $menu->add(Horde::applicationUrl('articles.php'), _("Articles"), 'datatree.png', $registry->getImageDir('horde'));

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
